//---------------------------------------------------------------------------
#define WIN32
#define WPCAP
#define HAVE_REMOTE
#include <vcl.h>
#include <WinPcap/pcap.h>
#ifndef WIN32
  #include <sys/socket.h>
  #include <netinet/in.h>
#else
  #include <Psdk/winsock2.h>
  #include <Psdk/ws2tcpip.h>
  #include <Psdk/windows.h>
#endif
#pragma hdrstop

//
// Function prototypes
//
void ifprint(pcap_if_t *d);
char *iptos(u_long in);
//---------------------------------------------------------------------------
int main()
{
  pcap_if_t *alldevs;
  pcap_if_t *d;
  char errbuf[PCAP_ERRBUF_SIZE+1];
  char source[PCAP_ERRBUF_SIZE+1];

  printf("Enter the device you want to list:\n"
            "rpcap://              ==> lists interfaces in the local machine\n"
            "rpcap://hostname:port ==> lists interfaces in a remote machine\n"
            "                          (rpcapd daemon must be up and running\n"
            "                           and it must accept 'null' authentication)\n"
            "file://foldername     ==> lists all pcap files in the give folder\n\n"
            "Enter your choice: ");

  gets(source);
  //
  // Retrieve the interfaces list.
  //
  if (pcap_findalldevs_ex(source, NULL, &alldevs, errbuf) == -1)
  {
    fprintf(stderr,"Error in pcap_findalldevs: %s\n",errbuf);
    exit(1);
  }
  //
  // Scan the list printing every entry.
  //
  for(d=alldevs;d;d=d->next)
  {
    ifprint(d);
  }
  Sleep(180000);
  return 1;
}
//---------------------------------------------------------------------------
//
//  Print all the available information on the given interface.
//
void ifprint(pcap_if_t *d)
{
  pcap_addr_t *a;
  //
  // Interface's name.
  //
  printf("%s\n",d->name);
  //
  // Interface's description.
  //
  if (d->description)
    printf("\tDescription: %s\n",d->description);
  //
  // Interface's loopback address.
  //
  printf("\tLoopback: %s\n",(d->flags & PCAP_IF_LOOPBACK)?"yes":"no");
  //
  // Interface's IP addresses.
  //
  for(a=d->addresses;a;a=a->next) {
    printf("\tAddress Family: #%d\n",a->addr->sa_family);
    switch(a->addr->sa_family)
    {
      case AF_INET:
        printf("\tAddress Family Name: AF_INET\n");
        if (a->addr)
          printf("\tAddress: %s\n",iptos(((struct sockaddr_in *)a->addr)->sin_addr.s_addr));
        if (a->netmask)
          printf("\tNetmask: %s\n",iptos(((struct sockaddr_in *)a->netmask)->sin_addr.s_addr));
        if (a->broadaddr)
          printf("\tBroadcast Address: %s\n",iptos(((struct sockaddr_in *)a->broadaddr)->sin_addr.s_addr));
        if (a->dstaddr)
          printf("\tDestination Address: %s\n",iptos(((struct sockaddr_in *)a->dstaddr)->sin_addr.s_addr));
        break;
     case AF_INET6:
       printf("\tAddress Family Name: AF_INET6\n");
        if (a->addr)
          printf("\tAddress: %s\n",iptos(((struct sockaddr_in *)a->addr)->sin_addr.s_addr));
        if (a->netmask)
          printf("\tNetmask: %s\n",iptos(((struct sockaddr_in *)a->netmask)->sin_addr.s_addr));
        if (a->broadaddr)
          printf("\tBroadcast Address: %s\n",iptos(((struct sockaddr_in *)a->broadaddr)->sin_addr.s_addr));
        if (a->dstaddr)
          printf("\tDestination Address: %s\n",iptos(((struct sockaddr_in *)a->dstaddr)->sin_addr.s_addr));
        break;
      default:
        printf("\tAddress Family Name: Unknown\n");
        break;
    }
  }
  printf("\n");
}
//---------------------------------------------------------------------------
//
// From tcptraceroute, convert a numeric IP address to a string
//
#define IPTOSBUFFERS    12
char *iptos(u_long in)
{
    static char output[IPTOSBUFFERS][3*4+3+1];
    static short which;
    u_char *p;
    p = (u_char *)&in;
    which = (which + 1 == IPTOSBUFFERS ? 0 : which + 1);
    sprintf(output[which], "%d.%d.%d.%d", p[0], p[1], p[2], p[3]);
    return output[which];
}
//---------------------------------------------------------------------------
